package benten.cat.glossary.engine.omegat.ui;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.preference.IPreferenceStore;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.BentenGlossaryDriver;
import benten.cat.glossary.core.BentenGlossaryDriverManager;
import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.data.AbstractBentenGlossaryDataTsvPlugin;
import benten.cat.glossary.engine.AbstractBentenGlossaryEnginePlugin;
import benten.cat.glossary.engine.omegat.DefaultGlossaryDriver;
import benten.cat.tm.engine.omegat.ui.messages.BentenGlossaryPluginMessages;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;

/**
 * Benten システムのデフォルトの用語集エンジン・プラグインである OmegaT 用語集プラグイン。
 * 
 * <UL>
 * <LI>benten.cat.glossary.engine.omegat を Eclipse にプラグします。
 * </UL>
 * 
 * ★基本設計「共通機能: 用語対訳集エンジン機能」に対応します。
 * 
 * @author YAMAMOTO Koji
 */
public class OmegatGlossaryPlugin extends AbstractBentenGlossaryEnginePlugin {
	/**
	 * Benten デフォルト用語集エンジン・プラグインのためのメッセージ。
	 */
	protected static final BentenGlossaryPluginMessages fMsg = new BentenGlossaryPluginMessages();

	private static final String DEFAULT_GLOSSARY = "Default Glossary"; //$NON-NLS-1$

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	@Override
	public BentenGlossaryDriver getDriver() {
		try {
			// 自らのドライバーを自己登録します。
			Class.forName("benten.cat.glossary.engine.omegat.DefaultGlossaryDriver"); //$NON-NLS-1$
		} catch (final ClassNotFoundException e) {
			log(fMsg.getGetDriverClassNotFoundException());
			log(e);
		}

		return BentenGlossaryDriverManager.getDriver(DefaultGlossaryDriver.DRIVER_NAME);
	}

	@Override
	public BentenGlossaryEngine getEngine() {
		if (fEngine == null) {
			fEngine = getDriver().getEngineInstance();
		}
		return fEngine;
	}

	@Override
	public void load() {
		getDefault().log(fMsg.getLoadI001());

		getEngine().unload();

		try {
			getEngine().loadTsv(getTsvDir());
		} catch (final IOException e) {
			getDefault().log(e);
		}
	}

	@SuppressWarnings("static-access")
	private File getTsvDir() throws IOException {
		File result = null;

		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.glossary.data.tsv"); //$NON-NLS-1$
		if (point == null) {
			return result;
		}

		final Map<String, AbstractBentenGlossaryDataTsvPlugin> glossaryMap = new HashMap<String, AbstractBentenGlossaryDataTsvPlugin>();

		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name"); //$NON-NLS-1$
					final Object obj = cfgElem.createExecutableExtension("class"); //$NON-NLS-1$
					if (obj instanceof AbstractBentenGlossaryDataTsvPlugin) {
						glossaryMap.put(name, (AbstractBentenGlossaryDataTsvPlugin) obj);
					}
				} catch (final CoreException e) {
					OmegatGlossaryPlugin.getDefault().log(e);
				}
			}
		}

		final AbstractBentenGlossaryDataTsvPlugin glossaryDataPlugin = glossaryMap.get(DEFAULT_GLOSSARY);
		if (glossaryDataPlugin == null) {
			return result;
		}

		final IPreferenceStore store = BentenPreference.getStore();
		final boolean useTmxLocation = store.getBoolean(Preference.USE_TSV_LOCATION.name());
		if (useTmxLocation) {
			final String tmxFolder = store.getString(Preference.TSV_LOCATION.name());
			result = new File(tmxFolder);
		} else {
			result = glossaryDataPlugin.getDefault().getFile("/glossary"); //$NON-NLS-1$
		}

		getDefault().log(fMsg.getGetTsvDirI001(result.toString()));
		return result;
	}
}
